@@ -118,21 +118,41 @@ class FpGroup(DefaultPrinting):
     def __contains__(self, g):
         return g in self.free_group
 
-    def subgroup(self, gens, C=None):
+    def subgroup(self, gens, C=None, homomorphism=False):
         '''
         Return the subgroup generated by `gens` using the
         Reidemeister-Schreier algorithm
+        homomorphism -- When set to True, return a dictionary containing the images
+                     of the presentation generators in the original group.
+
+        Examples
+        ========
+        >>> from sympy.combinatorics.fp_groups import (FpGroup, FpSubgroup)
+        >>> from sympy.combinatorics.free_groups import free_group
+        >>> F, x, y = free_group("x, y")
+        >>> f = FpGroup(F, [x**3, y**5, (x*y)**2])
+        >>> H = [x*y, x**-1*y**-1*x*y*x]
+        >>> K, T = f.subgroup(H, homomorphism=True)
+        >>> T(K.generators)
+        [x*y, x**-1*y**2*x**-1]
 
         '''
+
         if not all([isinstance(g, FreeGroupElement) for g in gens]):
             raise ValueError("Generators must be `FreeGroupElement`s")
         if not all([g.group == self.free_group for g in gens]):
                 raise ValueError("Given generators are not members of the group")
-        g, rels = reidemeister_presentation(self, gens, C=C)
+        if homomorphism:
+            g, rels, _gens = reidemeister_presentation(self, gens, C=C, homomorphism=True)
+        else:
+            g, rels = reidemeister_presentation(self, gens, C=C)
         if g:
             g = FpGroup(g[0].group, rels)
         else:
             g = FpGroup(free_group('')[0], [])
+        if homomorphism:
+            from sympy.combinatorics.homomorphisms import homomorphism
+            return g, homomorphism(g, self, g.generators, _gens, check=False)
         return g
 
     def coset_enumeration(self, H, strategy="relator_based", max_cosets=None,
@@ -1068,11 +1088,24 @@ def _simplification_technique_1(rels):
 ###############################################################################
 
 # Pg 175 [1]
-def define_schreier_generators(C):
+def define_schreier_generators(C, homomorphism=False):
+    '''
+    Arguments:
+    C -- Coset table.
+    homomorphism -- When set to True, return a dictionary containing the images
+                     of the presentation generators in the original group.
+    '''
     y = []
     gamma = 1
     f = C.fp_group
     X = f.generators
+    if homomorphism:
+        # `_gens` stores the elements of the parent group to
+        # to which the schreier generators correspond to.
+        _gens = {}
+        # compute the schreier Traversal
+        tau = {}
+        tau[0] = f.identity
     C.P = [[None]*len(C.A) for i in range(C.n)]
     for alpha, x in product(C.omega, C.A):
         beta = C.table[alpha][C.A_dict[x]]
@@ -1080,13 +1113,19 @@ def define_schreier_generators(C):
             C.P[alpha][C.A_dict[x]] = "<identity>"
             C.P[beta][C.A_dict_inv[x]] = "<identity>"
             gamma += 1
+            if homomorphism:
+                tau[beta] = tau[alpha]*x
         elif x in X and C.P[alpha][C.A_dict[x]] is None:
             y_alpha_x = '%s_%s' % (x, alpha)
             y.append(y_alpha_x)
             C.P[alpha][C.A_dict[x]] = y_alpha_x
+            if homomorphism:
+                _gens[y_alpha_x] = tau[alpha]*x*tau[beta]**-1
     grp_gens = list(free_group(', '.join(y)))
     C._schreier_free_group = grp_gens.pop(0)
     C._schreier_generators = grp_gens
+    if homomorphism:
+        C._schreier_gen_elem = _gens
     # replace all elements of P by, free group elements
     for i, j in product(range(len(C.P)), range(len(C.A))):
         # if equals "<identity>", replace by identity element
@@ -1209,11 +1248,13 @@ def elimination_technique_2(C):
     C._schreier_generators = gens
     return C._schreier_generators, C._reidemeister_relators
 
-def reidemeister_presentation(fp_grp, H, C=None):
+def reidemeister_presentation(fp_grp, H, C=None, homomorphism=False):
     """
     fp_group: A finitely presented group, an instance of FpGroup
     H: A subgroup whose presentation is to be found, given as a list
     of words in generators of `fp_grp`
+    homomorphism: When set to True, return a homomorphism from the subgroup
+                    to the parent group
 
     Examples
     ========
@@ -1250,7 +1291,7 @@ def reidemeister_presentation(fp_grp, H, C=None):
     if not C:
         C = coset_enumeration_r(fp_grp, H)
     C.compress(); C.standardize()
-    define_schreier_generators(C)
+    define_schreier_generators(C, homomorphism=homomorphism)
     reidemeister_relators(C)
     gens, rels = C._schreier_generators, C._reidemeister_relators
     gens, rels = simplify_presentation(gens, rels, change_gens=True)
@@ -1258,6 +1299,12 @@ def reidemeister_presentation(fp_grp, H, C=None):
     C.schreier_generators = tuple(gens)
     C.reidemeister_relators = tuple(rels)
 
+    if homomorphism:
+        _gens = []
+        for gen in gens:
+            _gens.append(C._schreier_gen_elem[str(gen)])
+        return C.schreier_generators, C.reidemeister_relators, _gens
+
     return C.schreier_generators, C.reidemeister_relators
 
 
