@@ -5,12 +5,12 @@ from .sympify import sympify, SympifyError
 from .cache import cacheit
 from .basic import Basic, Atom, preorder_traversal
 from .singleton import S
-from .expr import Expr, AtomicExpr
+from .expr import Expr, AtomicExpr, UnevaluatedExpr
 from .symbol import Symbol, Wild, Dummy, symbols, var
 from .numbers import Number, Float, Rational, Integer, NumberSymbol, \
     RealNumber, igcd, ilcm, seterr, E, I, nan, oo, pi, zoo, \
-    AlgebraicNumber, comp
-from .power import Pow, integer_nthroot
+    AlgebraicNumber, comp, mod_inverse
+from .power import Pow, integer_nthroot, integer_log
 from .mul import Mul, prod
 from .add import Add
 from .mod import Mod
@@ -32,3 +32,4 @@ from .evaluate import evaluate
 Catalan = S.Catalan
 EulerGamma = S.EulerGamma
 GoldenRatio = S.GoldenRatio
+TribonacciConstant = S.TribonacciConstant
