@@ -1,15 +1,18 @@
 """This tests sympy/core/basic.py with (ideally) no reference to subclasses
 of Basic or Atom."""
 
-from sympy.core.basic import Basic, Atom, preorder_traversal
-from sympy.core.singleton import S, Singleton
+import collections
+import sys
+
+from sympy.core.basic import Basic, Atom, preorder_traversal, as_Basic
+from sympy.core.singleton import S
 from sympy.core.symbol import symbols
-from sympy.core.compatibility import default_sort_key, with_metaclass
-
-from sympy import sin, Lambda, Q
+from sympy.core.compatibility import default_sort_key
 
+from sympy import sin, Lambda, Q, cos, gamma, Tuple
+from sympy.functions.elementary.exponential import exp
 from sympy.utilities.pytest import raises
-
+from sympy.core import I, pi
 
 b1 = Basic()
 b2 = Basic(b1)
@@ -35,6 +38,43 @@ def test_equality():
     assert Basic() != 0
     assert not(Basic() == 0)
 
+    class Foo(object):
+        """
+        Class that is unaware of Basic, and relies on both classes returning
+        the NotImplemented singleton for equivalence to evaluate to False.
+
+        """
+
+    b = Basic()
+    foo = Foo()
+
+    assert b != foo
+    assert foo != b
+    assert not b == foo
+    assert not foo == b
+
+    class Bar(object):
+        """
+        Class that considers itself equal to any instance of Basic, and relies
+        on Basic returning the NotImplemented singleton in order to achieve
+        a symmetric equivalence relation.
+
+        """
+        def __eq__(self, other):
+            if isinstance(other, Basic):
+                return True
+            return NotImplemented
+
+        def __ne__(self, other):
+            return not self == other
+
+    bar = Bar()
+
+    assert b == bar
+    assert bar == b
+    assert not b != bar
+    assert not bar != b
+
 
 def test_matches_basic():
     instances = [Basic(b1, b1, b2), Basic(b1, b2, b1), Basic(b2, b1, b1),
@@ -64,6 +104,9 @@ def test_subs():
     assert b21.subs([(b2, b1), (b1, b2)]) == Basic(b2, b2)
 
     assert b21.subs({b1: b2, b2: b1}) == Basic(b2, b2)
+    if sys.version_info >= (3, 4):
+        assert b21.subs(collections.ChainMap({b1: b2}, {b2: b1})) == Basic(b2, b2)
+    assert b21.subs(collections.OrderedDict([(b2, b1), (b1, b2)])) == Basic(b2, b2)
 
     raises(ValueError, lambda: b21.subs('bad arg'))
     raises(ValueError, lambda: b21.subs(b1, b2, b3))
@@ -97,32 +140,6 @@ def test_xreplace():
     raises(TypeError, lambda: b1.xreplace([b1, b2]))
 
 
-def test_Singleton():
-    global instantiated
-    instantiated = 0
-
-    class MySingleton(with_metaclass(Singleton, Basic)):
-        def __new__(cls):
-            global instantiated
-            instantiated += 1
-            return Basic.__new__(cls)
-
-    assert instantiated == 0
-    MySingleton() # force instantiation
-    assert instantiated == 1
-    assert MySingleton() is not Basic()
-    assert MySingleton() is MySingleton()
-    assert S.MySingleton is MySingleton()
-    assert instantiated == 1
-
-    class MySingleton_sub(MySingleton):
-        pass
-    assert instantiated == 1
-    MySingleton_sub()
-    assert instantiated == 2
-    assert MySingleton_sub() is not MySingleton()
-    assert MySingleton_sub() is MySingleton_sub()
-
 
 def test_preorder_traversal():
     expr = Basic(b21, b3)
@@ -173,6 +190,17 @@ def test_call():
 
     assert (Q.real & Q.positive).rcall(x) == Q.real(x) & Q.positive(x)
 
+
+def test_rewrite():
+    x, y, z = symbols('x y z')
+    a, b = symbols('a b')
+    f1 = sin(x) + cos(x)
+    assert f1.rewrite(cos,exp) == exp(I*x)/2 + sin(x) + exp(-I*x)/2
+    assert f1.rewrite([cos],sin) == sin(x) + sin(x + pi/2, evaluate=False)
+    f2 = sin(x) + cos(y)/gamma(z)
+    assert f2.rewrite(sin,exp) == -I*(exp(I*x) - exp(-I*x))/2 + cos(y)/gamma(z)
+
+
 def test_literal_evalf_is_number_is_zero_is_comparable():
     from sympy.integrals.integrals import Integral
     from sympy.core.symbol import symbols
@@ -181,11 +209,10 @@ def test_literal_evalf_is_number_is_zero_is_comparable():
     x = symbols('x')
     f = Function('f')
 
-    # the following should not be changed without a lot of dicussion
-    # `foo.is_number` should be equivalent to `not foo.free_symbols`
-    # it should not attempt anything fancy; see is_zero, is_constant
-    # and equals for more rigorous tests.
-    assert f(1).is_number is True
+    # issue 5033
+    assert f.is_number is False
+    # issue 6646
+    assert f(1).is_number is False
     i = Integral(0, (x, x, x))
     # expressions that are symbolically 0 can be difficult to prove
     # so in case there is some easy way to know if something is 0
@@ -202,3 +229,9 @@ def test_literal_evalf_is_number_is_zero_is_comparable():
     assert n.is_comparable is False
     assert n.n(2).is_comparable is False
     assert n.n(2).n(2).is_comparable
+
+
+def test_as_Basic():
+    assert as_Basic(1) is S.One
+    assert as_Basic(()) == Tuple()
+    raises(TypeError, lambda: as_Basic([]))
