@@ -2,13 +2,11 @@ from sympy import (
     symbols, log, ln, Float, nan, oo, zoo, I, pi, E, exp, Symbol,
     LambertW, sqrt, Rational, expand_log, S, sign, conjugate, refine,
     sin, cos, sinh, cosh, tanh, exp_polar, re, Function, simplify,
-    AccumBounds)
+    AccumBounds, MatrixSymbol)
+from sympy.abc import x, y, z
 
 
 def test_exp_values():
-
-    x, y = symbols('x,y')
-
     k = Symbol('k', integer=True)
 
     assert exp(nan) == nan
@@ -52,15 +50,11 @@ def test_exp_log():
     assert exp(x).inverse() == log
 
     y = Symbol("y", polar=True)
-    z = Symbol("z")
     assert log(exp_polar(z)) == z
     assert exp(log(y)) == y
 
 
 def test_exp_expand():
-    x = Symbol("x")
-    y = Symbol("y")
-
     e = exp(log(Rational(2))*(1 + x) - log(Rational(2))*x)
     assert e.expand() == 2
     assert exp(x + y) != exp(x)*exp(y)
@@ -68,8 +62,6 @@ def test_exp_expand():
 
 
 def test_exp__as_base_exp():
-    x, y = symbols('x,y')
-
     assert exp(x).as_base_exp() == (E, x)
     assert exp(2*x).as_base_exp() == (E, 2*x)
     assert exp(x*y).as_base_exp() == (E, x*y)
@@ -85,15 +77,15 @@ def test_exp__as_base_exp():
 
 
 def test_exp_infinity():
-    y = Symbol('y')
     assert exp(I*y) != nan
     assert refine(exp(I*oo)) == nan
     assert refine(exp(-I*oo)) == nan
     assert exp(y*I*oo) != nan
+    assert exp(zoo) == nan
 
 
 def test_exp_subs():
-    x, y = symbols('x,y')
+    x = Symbol('x')
     e = (exp(3*log(x), evaluate=False))  # evaluates to x**3
     assert e.subs(x**3, y**3) == e
     assert e.subs(x**2, 5) == e
@@ -112,22 +104,28 @@ def test_exp_subs():
 
 
 def test_exp_conjugate():
-    x = Symbol('x')
     assert conjugate(exp(x)) == exp(conjugate(x))
 
 
 def test_exp_rewrite():
-    x = symbols('x')
+    from sympy.concrete.summations import Sum
     assert exp(x).rewrite(sin) == sinh(x) + cosh(x)
     assert exp(x*I).rewrite(cos) == cos(x) + I*sin(x)
     assert exp(1).rewrite(cos) == sinh(1) + cosh(1)
     assert exp(1).rewrite(sin) == sinh(1) + cosh(1)
     assert exp(1).rewrite(sin) == sinh(1) + cosh(1)
     assert exp(x).rewrite(tanh) == (1 + tanh(x/2))/(1 - tanh(x/2))
+    assert exp(pi*I/4).rewrite(sqrt) == sqrt(2)/2 + sqrt(2)*I/2
+    assert exp(pi*I/3).rewrite(sqrt) == 1/2 + sqrt(3)*I/2
+
+    n = Symbol('n', integer=True)
+
+    assert Sum((exp(pi*I/2)/2)**n, (n, 0, oo)).rewrite(sqrt).doit() == 4/5 + 2*I/5
+    assert Sum((exp(pi*I/4)/2)**n, (n, 0, oo)).rewrite(sqrt).doit() == 1/(1 - sqrt(2)*(1 + I)/4)
+    assert Sum((exp(pi*I/3)/2)**n, (n, 0, oo)).rewrite(sqrt).doit() == 1/(3/4 - sqrt(3)*I/4)
 
 
 def test_exp_leading_term():
-    x = symbols('x')
     assert exp(x).as_leading_term(x) == 1
     assert exp(1/x).as_leading_term(x) == exp(1/x)
     assert exp(2 + x).as_leading_term(x) == exp(2)
@@ -136,6 +134,13 @@ def test_exp_taylor_term():
     x = symbols('x')
     assert exp(x).taylor_term(1, x) == x
     assert exp(x).taylor_term(3, x) == x**3/6
+    assert exp(x).taylor_term(4, x) == x**4/24
+
+
+def test_exp_MatrixSymbol():
+    A = MatrixSymbol("A", 2, 2)
+    assert exp(A).has(exp)
+
 
 def test_log_values():
     assert log(nan) == nan
@@ -188,15 +193,13 @@ def test_log_base():
     assert log(3**3, 3) == 3
     assert log(5, 1) == zoo
     assert log(1, 1) == nan
-    assert log(Rational(2, 3), 10) == (-log(3) + log(2))/log(10)
+    assert log(Rational(2, 3), 10) == log(S(2)/3)/log(10)
     assert log(Rational(2, 3), Rational(1, 3)) == -log(2)/log(3) + 1
     assert log(Rational(2, 3), Rational(2, 5)) == \
-        (-log(3) + log(2))/(-log(5) + log(2))
+        log(S(2)/3)/log(S(2)/5)
 
 
 def test_log_symbolic():
-    x, y = symbols('x,y')
-
     assert log(x, exp(1)) == log(x)
     assert log(exp(x)) != x
 
@@ -235,7 +238,6 @@ def test_log_symbolic():
 
 
 def test_exp_assumptions():
-    x = Symbol('x')
     r = Symbol('r', real=True)
     i = Symbol('i', imaginary=True)
     for e in exp, exp_polar:
@@ -287,7 +289,6 @@ def test_log_assumptions():
 
 
 def test_log_hashing():
-    x = Symbol("y")
     assert x != log(log(x))
     assert hash(x) != hash(log(log(x)))
     assert log(x) != log(log(log(x)))
@@ -297,7 +298,6 @@ def test_log_hashing():
     e = 1/log(log(x) + log(log(log(x))))
     assert e.base.func is log
 
-    x = Symbol("x")
     e = log(log(x))
     assert e.func is log
     assert not x.func is log
@@ -348,13 +348,18 @@ def test_log_simplify():
     assert log(x**2).expand() == 2*log(x)
     assert expand_log(log(x**(2 + log(2)))) == (2 + log(2))*log(x)
 
+    z = Symbol('z')
+    assert log(sqrt(z)).expand() == log(z)/2
+    assert expand_log(log(z**(log(2) - 1))) == (log(2) - 1)*log(z)
+    assert log(z**(-1)).expand() != -log(z)
+    assert log(z**(x/(x+1))).expand() == x*log(z)/(x + 1)
+
 
 def test_log_AccumBounds():
     assert log(AccumBounds(1, E)) == AccumBounds(0, 1)
 
 
 def test_lambertw():
-    x = Symbol('x')
     k = Symbol('k')
 
     assert LambertW(x, 0) == LambertW(x)
@@ -405,7 +410,6 @@ def test_issue_5673():
 
 def test_exp_expand_NC():
     A, B, C = symbols('A,B,C', commutative=False)
-    x, y, z = symbols('x,y,z')
 
     assert exp(A + B).expand() == exp(A + B)
     assert exp(A + B + C).expand() == exp(A + B + C)
@@ -414,7 +418,6 @@ def test_exp_expand_NC():
 
 
 def test_as_numer_denom():
-    from sympy.abc import x
     n = symbols('n', negative=True)
     assert exp(x).as_numer_denom() == (exp(x), 1)
     assert exp(-x).as_numer_denom() == (1, exp(x))
@@ -429,9 +432,10 @@ def test_as_numer_denom():
 
 def test_polar():
     x, y = symbols('x y', polar=True)
-    z = Symbol('z')
 
     assert abs(exp_polar(I*4)) == 1
+    assert abs(exp_polar(0)) == 1
+    assert abs(exp_polar(2 + 3*I)) == exp(2)
     assert exp_polar(I*10).n() == exp_polar(I*10)
 
     assert log(exp_polar(z)) == z
@@ -461,7 +465,6 @@ def test_log_product():
 
 
 def test_issue_8866():
-    x = Symbol('x')
     assert simplify(log(x, 10, evaluate=False)) == simplify(log(x, 10))
     assert expand_log(log(x, 10, evaluate=False)) == expand_log(log(x, 10))
 
