@@ -1,4 +1,4 @@
-"""Integration functions that integrates a sympy expression.
+"""Integration functions that integrate a sympy expression.
 
     Examples
     ========
@@ -23,3 +23,4 @@ from .transforms import (mellin_transform, inverse_mellin_transform,
                         CosineTransform, InverseCosineTransform,
                         hankel_transform, inverse_hankel_transform,
                         HankelTransform, InverseHankelTransform)
+from .singularityfunctions import singularityintegrate
