@@ -1,36 +1,17 @@
 from sympy import (pi, sin, cos, Symbol, Integral, Sum, sqrt, log,
-                   oo, LambertW, I, meijerg, exp_polar, Max, Piecewise)
+                   oo, LambertW, I, meijerg, exp_polar, Max, Piecewise, And)
 from sympy.plotting import (plot, plot_parametric, plot3d_parametric_line,
                             plot3d, plot3d_parametric_surface)
-from sympy.plotting.plot import unset_show
+from sympy.plotting.plot import unset_show, plot_contour
+from sympy.utilities import lambdify as lambdify_
 from sympy.utilities.pytest import skip, raises
 from sympy.plotting.experimental_lambdify import lambdify
 from sympy.external import import_module
-from sympy.core.decorators import wraps
 
 from tempfile import NamedTemporaryFile
 import os
-import sys
 import warnings
 
-class MockPrint(object):
-
-    def write(self, s):
-        pass
-
-    def flush(self):
-        pass
-
-    encoding = 'utf-8'
-
-def disable_print(func, *args, **kwargs):
-    @wraps(func)
-    def wrapper(*args, **kwargs):
-        sys.stdout = MockPrint()
-        func(*args, **kwargs)
-        sys.stdout = sys.__stdout__
-    return wrapper
-
 unset_show()
 
 
@@ -48,9 +29,14 @@ class TmpFileManager:
 
     @classmethod
     def cleanup(cls):
-        map(os.remove, cls.tmp_files)
+        for file in cls.tmp_files:
+            try:
+                os.remove(file)
+            except OSError:
+                # If the file doesn't exist, for instance, if the test failed.
+                pass
 
-def plot_and_save(name):
+def plot_and_save_1(name):
     tmp_file = TmpFileManager.tmp_file
 
     x = Symbol('x')
@@ -97,10 +83,36 @@ def plot_and_save(name):
 
     raises(ValueError, lambda: plot(x, y))
 
-    p = plot(Piecewise((1, x > 0), (0, True)),(x,-1,1))
+    #Piecewise plots
+    p = plot(Piecewise((1, x > 0), (0, True)), (x, -1, 1))
     p.save(tmp_file('%s_plot_piecewise' % name))
     p._backend.close()
 
+    p = plot(Piecewise((x, x < 1), (x**2, True)), (x, -3, 3))
+    p.save(tmp_file('%s_plot_piecewise_2' % name))
+    p._backend.close()
+
+    # test issue 7471
+    p1 = plot(x)
+    p2 = plot(3)
+    p1.extend(p2)
+    p.save(tmp_file('%s_horizontal_line' % name))
+    p._backend.close()
+
+    # test issue 10925
+    f = Piecewise((-1, x < -1), (x, And(-1 <= x, x < 0)), \
+        (x**2, And(0 <= x, x < 1)), (x**3, x >= 1))
+    p = plot(f, (x, -3, 3))
+    p.save(tmp_file('%s_plot_piecewise_3' % name))
+    p._backend.close()
+
+def plot_and_save_2(name):
+    tmp_file = TmpFileManager.tmp_file
+
+    x = Symbol('x')
+    y = Symbol('y')
+    z = Symbol('z')
+
     #parametric 2d plots.
     #Single plot with default range.
     plot_parametric(sin(x), cos(x)).save(tmp_file())
@@ -172,6 +184,28 @@ def plot_and_save(name):
     p.save(tmp_file('%s_parametric_surface' % name))
     p._backend.close()
 
+    # Single Contour plot.
+    p = plot_contour(sin(x)*sin(y), (x, -5, 5), (y, -5, 5))
+    p.save(tmp_file('%s_contour_plot' % name))
+    p._backend.close()
+
+    # Multiple Contour plots with same range.
+    p = plot_contour(x**2 + y**2, x**3 + y**3, (x, -5, 5), (y, -5, 5))
+    p.save(tmp_file('%s_contour_plot' % name))
+    p._backend.close()
+
+    # Multiple Contour plots with different range.
+    p = plot_contour((x**2 + y**2, (x, -5, 5), (y, -5, 5)), (x**3 + y**3, (x, -3, 3), (y, -3, 3)))
+    p.save(tmp_file('%s_contour_plot' % name))
+    p._backend.close()
+
+def plot_and_save_3(name):
+    tmp_file = TmpFileManager.tmp_file
+
+    x = Symbol('x')
+    y = Symbol('y')
+    z = Symbol('z')
+
     ###
     # Examples from the 'colors' notebook
     ###
@@ -199,7 +233,7 @@ def plot_and_save(name):
              cos(x) + 0.1*cos(x)*cos(7*x),
         0.1*sin(7*x),
         (x, 0, 2*pi))
-    p[0].line_color = lambda a: sin(4*a)
+    p[0].line_color = lambdify_(x, sin(4*x))
     p.save(tmp_file('%s_colors_3d_line_arity1' % name))
     p[0].line_color = lambda a, b: b
     p.save(tmp_file('%s_colors_3d_line_arity2' % name))
@@ -214,7 +248,7 @@ def plot_and_save(name):
     p.save(tmp_file('%s_colors_surface_arity2' % name))
     p[0].surface_color = lambda a, b, c: c
     p.save(tmp_file('%s_colors_surface_arity3a' % name))
-    p[0].surface_color = lambda a, b, c: sqrt((a - 3*pi)**2 + b**2)
+    p[0].surface_color = lambdify_((x, y, z), sqrt((x - 3*pi)**2 + y**2))
     p.save(tmp_file('%s_colors_surface_arity3b' % name))
     p._backend.close()
 
@@ -224,10 +258,17 @@ def plot_and_save(name):
     p.save(tmp_file('%s_colors_param_surf_arity1' % name))
     p[0].surface_color = lambda a, b: a*b
     p.save(tmp_file('%s_colors_param_surf_arity2' % name))
-    p[0].surface_color = lambda a, b, c: sqrt(a**2 + b**2 + c**2)
+    p[0].surface_color = lambdify_((x, y, z), sqrt(x**2 + y**2 + z**2))
     p.save(tmp_file('%s_colors_param_surf_arity3' % name))
     p._backend.close()
 
+def plot_and_save_4(name):
+    tmp_file = TmpFileManager.tmp_file
+
+    x = Symbol('x')
+    y = Symbol('y')
+    z = Symbol('z')
+
     ###
     # Examples from the 'advanced' notebook
     ###
@@ -244,9 +285,16 @@ def plot_and_save(name):
         p.save(tmp_file('%s_advanced_integral' % name))
         p._backend.close()
         # Make sure no other warnings were raised
-        assert len(w) == 1
-        assert issubclass(w[-1].category, UserWarning)
-        assert "The evaluation of the expression is problematic" in str(w[0].message)
+        for i in w:
+            assert issubclass(i.category, UserWarning)
+            assert "The evaluation of the expression is problematic" in str(i.message)
+
+def plot_and_save_5(name):
+    tmp_file = TmpFileManager.tmp_file
+
+    x = Symbol('x')
+    y = Symbol('y')
+    z = Symbol('z')
 
     s = Sum(1/x**y, (x, 1, oo))
     p = plot(s, (y, 2, 10))
@@ -259,6 +307,13 @@ def plot_and_save(name):
     p.save(tmp_file('%s_advanced_fin_sum' % name))
     p._backend.close()
 
+def plot_and_save_6(name):
+    tmp_file = TmpFileManager.tmp_file
+
+    x = Symbol('x')
+    y = Symbol('y')
+    z = Symbol('z')
+
     ###
     # Test expressions that can not be translated to np and generate complex
     # results.
@@ -273,12 +328,72 @@ def plot_and_save(name):
             + meijerg(((1/2,), ()), ((5, 0, 1/2), ()),
                 5*x**2 * exp_polar(I*pi)/2)) / (48 * pi), (x, 1e-6, 1e-2)).save(tmp_file())
 
-def test_matplotlib():
+def test_matplotlib_1():
 
     matplotlib = import_module('matplotlib', min_module_version='1.1.0', catch=(RuntimeError,))
     if matplotlib:
         try:
-            plot_and_save('test')
+            plot_and_save_1('test')
+        finally:
+            # clean up
+            TmpFileManager.cleanup()
+    else:
+        skip("Matplotlib not the default backend")
+
+def test_matplotlib_2():
+
+    matplotlib = import_module('matplotlib', min_module_version='1.1.0', catch=(RuntimeError,))
+    if matplotlib:
+        try:
+            plot_and_save_2('test')
+        finally:
+            # clean up
+            TmpFileManager.cleanup()
+    else:
+        skip("Matplotlib not the default backend")
+
+def test_matplotlib_3():
+
+    matplotlib = import_module('matplotlib', min_module_version='1.1.0', catch=(RuntimeError,))
+    if matplotlib:
+        try:
+            plot_and_save_3('test')
+        finally:
+            # clean up
+            TmpFileManager.cleanup()
+    else:
+        skip("Matplotlib not the default backend")
+
+def test_matplotlib_4():
+
+    matplotlib = import_module('matplotlib', min_module_version='1.1.0', catch=(RuntimeError,))
+    if matplotlib:
+        try:
+            plot_and_save_4('test')
+        finally:
+            # clean up
+            TmpFileManager.cleanup()
+    else:
+        skip("Matplotlib not the default backend")
+
+def test_matplotlib_5():
+
+    matplotlib = import_module('matplotlib', min_module_version='1.1.0', catch=(RuntimeError,))
+    if matplotlib:
+        try:
+            plot_and_save_5('test')
+        finally:
+            # clean up
+            TmpFileManager.cleanup()
+    else:
+        skip("Matplotlib not the default backend")
+
+def test_matplotlib_6():
+
+    matplotlib = import_module('matplotlib', min_module_version='1.1.0', catch=(RuntimeError,))
+    if matplotlib:
+        try:
+            plot_and_save_6('test')
         finally:
             # clean up
             TmpFileManager.cleanup()
@@ -299,8 +414,12 @@ def test_experimental_lambify():
     f = lambdify([x], x + 1)
     assert f(1) == 2
 
-@disable_print
+
 def test_append_issue_7140():
+    matplotlib = import_module('matplotlib', min_module_version='1.1.0', catch=(RuntimeError,))
+    if not matplotlib:
+        skip("Matplotlib not the default backend")
+
     x = Symbol('x')
     p1 = plot(x)
     p2 = plot(x**2)
