@@ -226,7 +226,7 @@ def _roots_quartic_euler(p, q, r, a):
     -sqrt(32*sqrt(5)/125 + 16/5) + 4*sqrt(5)/5
     """
     # solve the resolvent equation
-    x = Symbol('x')
+    x = Dummy('x')
     eq = 64*x**3 + 32*p*x**2 + (4*p**2 - 16*r)*x - q**2
     xsols = list(roots(Poly(eq, x), cubics=False).keys())
     xsols = [sol for sol in xsols if sol.is_rational and sol.is_nonzero]
