@@ -34,22 +34,25 @@ more information on each (run help(pde)):
 """
 from __future__ import print_function, division
 
+from itertools import combinations_with_replacement
 from sympy.simplify import simplify
 from sympy.core import Add, S
-from sympy.core.compatibility import (reduce, combinations_with_replacement,
-    is_sequence, range)
+from sympy.core.compatibility import (reduce, is_sequence, range)
 from sympy.core.function import Function, expand, AppliedUndef, Subs
 from sympy.core.relational import Equality, Eq
 from sympy.core.symbol import Symbol, Wild, symbols
 from sympy.functions import exp
 from sympy.integrals.integrals import Integral
 from sympy.utilities.iterables import has_dups
+from sympy.utilities.misc import filldedent
 
 from sympy.solvers.deutils import _preprocess, ode_order, _desolve
 from sympy.solvers.solvers import solve
 from sympy.simplify.radsimp import collect
+
 import operator
 
+
 allhints = (
     "1st_linear_constant_coeff_homogeneous",
     "1st_linear_constant_coeff",
@@ -57,6 +60,7 @@ allhints = (
     "1st_linear_variable_coeff"
     )
 
+
 def pdsolve(eq, func=None, hint='default', dict=False, solvefun=None, **kwargs):
     """
     Solves any (supported) kind of partial differential equation.
@@ -193,10 +197,11 @@ def pdsolve(eq, func=None, hint='default', dict=False, solvefun=None, **kwargs):
         return _helper_simplify(eq, hints['hint'],
             hints['func'], hints['order'], hints[hints['hint']], solvefun)
 
+
 def _helper_simplify(eq, hint, func, order, match, solvefun):
     """Helper function of pdsolve that calls the respective
     pde functions to solve for the partial differential
-    equations. This minimises the computation in
+    equations. This minimizes the computation in
     calling _desolve multiple times.
     """
 
@@ -208,6 +213,7 @@ def _helper_simplify(eq, hint, func, order, match, solvefun):
     return _handle_Integral(solvefunc(eq, func, order,
         match, solvefun), func, order, hint)
 
+
 def _handle_Integral(expr, func, order, hint):
     r"""
     Converts a solution with integrals in it into an actual solution.
@@ -384,6 +390,7 @@ def classify_pde(eq, func=None, dict=False, **kwargs):
     else:
         return tuple(retlist)
 
+
 def checkpdesol(pde, sol, func=None, solve_for_func=True):
     """
     Checks if the given solution satisfies the partial differential
@@ -445,44 +452,38 @@ def checkpdesol(pde, sol, func=None, solve_for_func=True):
     # If the given solution is in the form of a list or a set
     # then return a list or set of tuples.
     if is_sequence(sol, set):
-        return type(sol)([checkpdesol(pde, i, solve_for_func=solve_for_func) for i in sol])
+        return type(sol)([checkpdesol(
+            pde, i, func=func,
+            solve_for_func=solve_for_func) for i in sol])
 
     # Convert solution into an equation
     if not isinstance(sol, Equality):
         sol = Eq(func, sol)
+    elif sol.rhs == func:
+        sol = sol.reversed
 
     # Try solving for the function
-    if solve_for_func and not (sol.lhs == func and not sol.rhs.has(func)) and not \
-            (sol.rhs == func and not sol.lhs.has(func)):
-        try:
-            solved = solve(sol, func)
-            if not solved:
-                raise NotImplementedError
-        except NotImplementedError:
-            pass
-        else:
+    solved = sol.lhs == func and not sol.rhs.has(func)
+    if solve_for_func and not solved:
+        solved = solve(sol, func)
+        if solved:
             if len(solved) == 1:
-                result = checkpdesol(pde, Eq(func, solved[0]),
-                    order=order, solve_for_func=False)
+                return checkpdesol(pde, Eq(func, solved[0]),
+                    func=func, solve_for_func=False)
             else:
-                result = checkpdesol(pde, [Eq(func, t) for t in solved],
-                order=order, solve_for_func=False)
+                return checkpdesol(pde, [Eq(func, t) for t in solved],
+                    func=func, solve_for_func=False)
 
-    # The first method includes direct substitution of the solution in
-    # the PDE and simplifying.
-    pde = pde.lhs - pde.rhs
+    # try direct substitution of the solution into the PDE and simplify
     if sol.lhs == func:
-        s = pde.subs(func, sol.rhs).doit()
-    elif sol.rhs == func:
-        s = pde.subs(func, sol.lhs).doit()
-    if s:
-        ss = simplify(s)
-        if ss:
-            return False, ss
-        else:
-            return True, 0
-    else:
-        return True, 0
+        pde = pde.lhs - pde.rhs
+        s = simplify(pde.subs(func, sol.rhs).doit())
+        return s is S.Zero, s
+
+    raise NotImplementedError(filldedent('''
+        Unable to test if %s is a solution to %s.''' % (sol, pde)))
+
+
 
 def pde_1st_linear_constant_coeff_homogeneous(eq, func, order, match, solvefun):
     r"""
@@ -553,6 +554,7 @@ def pde_1st_linear_constant_coeff_homogeneous(eq, func, order, match, solvefun):
     d = match[match['d']]
     return Eq(f(x,y), exp(-S(d)/(b**2 + c**2)*(b*x + c*y))*solvefun(c*x - b*y))
 
+
 def pde_1st_linear_constant_coeff(eq, func, order, match, solvefun):
     r"""
     Solves a first order linear partial differential equation
@@ -656,6 +658,7 @@ def pde_1st_linear_constant_coeff(eq, func, order, match, solvefun):
     return Eq(f(x,y), Subs(expterm*(functerm + genterm),
         (eta, xi), (c*x - b*y, b*x + c*y)))
 
+
 def pde_1st_linear_variable_coeff(eq, func, order, match, solvefun):
     r"""
     Solves a first order linear partial differential equation
@@ -749,7 +752,7 @@ def pde_1st_linear_variable_coeff(eq, func, order, match, solvefun):
         # The PDE reduces to b*(u.diff(x)) + d*u = e, which is a linear ODE in x
         plode = f(x).diff(x)*b + d*f(x) - e
         sol = dsolve(plode, f(x))
-        syms = sol.free_symbols - plode.free_symbols - set([x, y])
+        syms = sol.free_symbols - plode.free_symbols - {x, y}
         rhs = _simplify_variable_coeff(sol.rhs, syms, solvefun, y)
         return Eq(f(x, y), rhs)
 
@@ -758,7 +761,7 @@ def pde_1st_linear_variable_coeff(eq, func, order, match, solvefun):
         # The PDE reduces to c*(u.diff(y)) + d*u = e, which is a linear ODE in y
         plode = f(y).diff(y)*c + d*f(y) - e
         sol = dsolve(plode, f(y))
-        syms = sol.free_symbols - plode.free_symbols - set([x, y])
+        syms = sol.free_symbols - plode.free_symbols - {x, y}
         rhs = _simplify_variable_coeff(sol.rhs, syms, solvefun, x)
         return Eq(f(x, y), rhs)
 
@@ -767,7 +770,7 @@ def pde_1st_linear_variable_coeff(eq, func, order, match, solvefun):
     sol = dsolve(dummy(x).diff(x) - h, dummy(x))
     if isinstance(sol, list):
         sol = sol[0]
-    solsym = sol.free_symbols - h.free_symbols - set([x, y])
+    solsym = sol.free_symbols - h.free_symbols - {x, y}
     if len(solsym) == 1:
         solsym = solsym.pop()
         etat = (solve(sol, solsym)[0]).subs(dummy(x), y)
@@ -776,7 +779,7 @@ def pde_1st_linear_variable_coeff(eq, func, order, match, solvefun):
         final = (dsolve(deq, f(x), hint='1st_linear')).rhs
         if isinstance(final, list):
             final = final[0]
-        finsyms = final.free_symbols - deq.free_symbols - set([x, y])
+        finsyms = final.free_symbols - deq.free_symbols - {x, y}
         rhs = _simplify_variable_coeff(final, finsyms, solvefun, etat)
         return Eq(f(x, y), rhs)
 
@@ -784,6 +787,7 @@ def pde_1st_linear_variable_coeff(eq, func, order, match, solvefun):
         raise NotImplementedError("Cannot solve the partial differential equation due"
             " to inability of constantsimp")
 
+
 def _simplify_variable_coeff(sol, syms, func, funcarg):
     r"""
     Helper function to replace constants by functions in 1st_linear_variable_coeff
@@ -801,6 +805,7 @@ def _simplify_variable_coeff(sol, syms, func, funcarg):
 
     return simplify(final.subs(eta, funcarg))
 
+
 def pde_separate(eq, fun, sep, strategy='mul'):
     """Separate variables in partial differential equation either by additive
     or multiplicative separation approach. It tries to rewrite an equation so
@@ -830,7 +835,7 @@ def pde_separate(eq, fun, sep, strategy='mul'):
 
     >>> eq = Eq(D(u(x, t), x, 2), D(u(x, t), t, 2))
     >>> pde_separate(eq, u(x, t), [X(x), T(t)], strategy='mul')
-    [Derivative(X(x), x, x)/X(x), Derivative(T(t), t, t)/T(t)]
+    [Derivative(X(x), (x, 2))/X(x), Derivative(T(t), (t, 2))/T(t)]
 
     See Also
     ========
@@ -843,11 +848,14 @@ def pde_separate(eq, fun, sep, strategy='mul'):
     elif strategy == 'mul':
         do_add = False
     else:
-        assert ValueError('Unknown strategy: %s' % strategy)
+        raise ValueError('Unknown strategy: %s' % strategy)
 
     if isinstance(eq, Equality):
         if eq.rhs != 0:
             return pde_separate(Eq(eq.lhs - eq.rhs), fun, sep, strategy)
+    else:
+        return pde_separate(Eq(eq, 0), fun, sep, strategy)
+
     if eq.rhs != 0:
         raise ValueError("Value should be 0")
 
@@ -932,7 +940,7 @@ def pde_separate_mul(eq, fun, sep):
 
     >>> eq = Eq(D(u(x, y), x, 2), D(u(x, y), y, 2))
     >>> pde_separate_mul(eq, u(x, y), [X(x), Y(y)])
-    [Derivative(X(x), x, x)/X(x), Derivative(Y(y), y, y)/Y(y)]
+    [Derivative(X(x), (x, 2))/X(x), Derivative(Y(y), (y, 2))/Y(y)]
 
     """
     return pde_separate(eq, fun, sep, strategy='mul')
