@@ -19,7 +19,7 @@ where `k > 0` and `a_{i}(n)` are polynomials in `n`. To use
 ``rsolve_X`` we need to put all coefficients in to a list ``L`` of
 `k+1` elements the following way:
 
-    ``L = [ a_{0}(n), ..., a_{k-1}(n), a_{k}(n) ]``
+    ``L = [a_{0}(n), ..., a_{k-1}(n), a_{k}(n)]``
 
 where ``L[i]``, for `i=0, \ldots, k`, maps to
 `a_{i}(n) y(n+i)` (`y(n+i)` is implicit).
@@ -42,8 +42,8 @@ n**4 - 2*n**3 + n**2 - 1/30
 
 For the sake of completeness, `f(n)` can be:
 
-    [1] a polynomial              -> rsolve_poly
-    [2] a rational function       -> rsolve_ratio
+    [1] a polynomial               -> rsolve_poly
+    [2] a rational function        -> rsolve_ratio
     [3] a hypergeometric function  -> rsolve_hyper
 """
 from __future__ import print_function, division
@@ -129,12 +129,12 @@ def rsolve_poly(coeffs, f, n, **hints):
 
     r = len(coeffs) - 1
 
-    coeffs = [ Poly(coeff, n) for coeff in coeffs ]
+    coeffs = [Poly(coeff, n) for coeff in coeffs]
 
-    polys = [ Poly(0, n) ] * (r + 1)
-    terms = [ (S.Zero, S.NegativeInfinity) ] *(r + 1)
+    polys = [Poly(0, n)]*(r + 1)
+    terms = [(S.Zero, S.NegativeInfinity)]*(r + 1)
 
-    for i in range(0, r + 1):
+    for i in range(r + 1):
         for j in range(i, r + 1):
             polys[i] += coeffs[j]*binomial(j, i)
 
@@ -157,7 +157,7 @@ def rsolve_poly(coeffs, f, n, **hints):
 
     degree_poly = S.Zero
 
-    for i in range(0, r + 1):
+    for i in range(r + 1):
         if terms[i][1] - i == b:
             degree_poly += terms[i][0]*FallingFactorial(x, i)
 
@@ -189,17 +189,17 @@ def rsolve_poly(coeffs, f, n, **hints):
         C = []
         y = E = S.Zero
 
-        for i in range(0, N + 1):
+        for i in range(N + 1):
             C.append(Symbol('C' + str(i)))
             y += C[i] * n**i
 
-        for i in range(0, r + 1):
+        for i in range(r + 1):
             E += coeffs[i].as_expr()*y.subs(n, n + i)
 
         solutions = solve_undetermined_coeffs(E - f, C, n)
 
         if solutions is not None:
-            C = [ c for c in C if (c not in solutions) ]
+            C = [c for c in C if (c not in solutions)]
             result = y.subs(solutions)
         else:
             return None  # TBD
@@ -241,8 +241,8 @@ def rsolve_poly(coeffs, f, n, **hints):
 
             alpha[i] = S.Zero
 
-            for j in range(0, A + 1):
-                for k in range(0, d + 1):
+            for j in range(A + 1):
+                for k in range(d + 1):
                     B = binomial(k, i + j)
                     D = _delta(polys[j].as_expr(), k)
 
@@ -260,12 +260,12 @@ def rsolve_poly(coeffs, f, n, **hints):
 
                     B = alpha[k - A].subs(x, i - k)
 
-                    for j in range(0, A):
+                    for j in range(A):
                         v[j] += B * V[i - k, j]
 
                 denom = alpha[-A].subs(x, i)
 
-                for j in range(0, A):
+                for j in range(A):
                     V[i, j] = -v[j] / denom
         else:
             G = _zero_vector(U)
@@ -280,14 +280,14 @@ def rsolve_poly(coeffs, f, n, **hints):
 
                     B = alpha[k - A].subs(x, i - k)
 
-                    for j in range(0, A):
+                    for j in range(A):
                         v[j] += B * V[i - k, j]
 
                     g += B * G[i - k]
 
                 denom = alpha[-A].subs(x, i)
 
-                for j in range(0, A):
+                for j in range(A):
                     V[i, j] = -v[j] / denom
 
                 G[i] = (_delta(f, i - A) - g) / denom
@@ -297,20 +297,20 @@ def rsolve_poly(coeffs, f, n, **hints):
         for i in range(1, U):
             P[i] = (P[i - 1] * (n - a - i + 1)/i).expand()
 
-        for i in range(0, A):
-            Q[i] = Add(*[ (v*p).expand() for v, p in zip(V[:, i], P) ])
+        for i in range(A):
+            Q[i] = Add(*[(v*p).expand() for v, p in zip(V[:, i], P)])
 
         if not homogeneous:
-            h = Add(*[ (g*p).expand() for g, p in zip(G, P) ])
+            h = Add(*[(g*p).expand() for g, p in zip(G, P)])
 
-        C = [ Symbol('C' + str(i)) for i in range(0, A) ]
+        C = [Symbol('C' + str(i)) for i in range(A)]
 
-        g = lambda i: Add(*[ c*_delta(q, i) for c, q in zip(C, Q) ])
+        g = lambda i: Add(*[c*_delta(q, i) for c, q in zip(C, Q)])
 
         if homogeneous:
-            E = [ g(i) for i in range(N + 1, U) ]
+            E = [g(i) for i in range(N + 1, U)]
         else:
-            E = [ g(i) + _delta(h, i) for i in range(N + 1, U) ]
+            E = [g(i) + _delta(h, i) for i in range(N + 1, U)]
 
         if E != []:
             solutions = solve(E, *C)
@@ -433,17 +433,17 @@ def rsolve_ratio(coeffs, f, n, **hints):
             A = quo(A, d, n)
             B = quo(B, d.subs(n, n - i), n)
 
-            C *= Mul(*[ d.subs(n, n - j) for j in range(0, i + 1) ])
+            C *= Mul(*[d.subs(n, n - j) for j in range(i + 1)])
 
-        denoms = [ C.subs(n, n + i) for i in range(0, r + 1) ]
+        denoms = [C.subs(n, n + i) for i in range(r + 1)]
 
-        for i in range(0, r + 1):
+        for i in range(r + 1):
             g = gcd(coeffs[i], denoms[i], n)
 
             numers[i] = quo(coeffs[i], g, n)
             denoms[i] = quo(denoms[i], g, n)
 
-        for i in range(0, r + 1):
+        for i in range(r + 1):
             numers[i] *= Mul(*(denoms[:i] + denoms[i + 1:]))
 
         result = rsolve_poly(numers, f * Mul(*denoms), n, **hints)
@@ -544,7 +544,7 @@ def rsolve_hyper(coeffs, f, n, **hints):
 
         for i, g in enumerate(inhomogeneous):
             coeff, polys = S.One, coeffs[:]
-            denoms = [ S.One ] * (r + 1)
+            denoms = [S.One]*(r + 1)
 
             s = hypersimp(g, n)
 
@@ -556,7 +556,7 @@ def rsolve_hyper(coeffs, f, n, **hints):
                 polys[j] *= p
                 denoms[j] = q
 
-            for j in range(0, r + 1):
+            for j in range(r + 1):
                 polys[j] *= Mul(*(denoms[:j] + denoms[j + 1:]))
 
             R = rsolve_poly(polys, Mul(*denoms), n)
@@ -574,10 +574,10 @@ def rsolve_hyper(coeffs, f, n, **hints):
 
     p, q = coeffs[0], coeffs[r].subs(n, n - r + 1)
 
-    p_factors = [ z for z in roots(p, n).keys() ]
-    q_factors = [ z for z in roots(q, n).keys() ]
+    p_factors = [z for z in roots(p, n).keys()]
+    q_factors = [z for z in roots(q, n).keys()]
 
-    factors = [ (S.One, S.One) ]
+    factors = [(S.One, S.One)]
 
     for p in p_factors:
         for q in q_factors:
@@ -586,8 +586,8 @@ def rsolve_hyper(coeffs, f, n, **hints):
             else:
                 factors += [(n - p, n - q)]
 
-    p = [ (n - p, S.One) for p in p_factors ]
-    q = [ (S.One, n - q) for q in q_factors ]
+    p = [(n - p, S.One) for p in p_factors]
+    q = [(S.One, n - q) for q in q_factors]
 
     factors = p + factors + q
 
@@ -595,9 +595,9 @@ def rsolve_hyper(coeffs, f, n, **hints):
         polys, degrees = [], []
         D = A*B.subs(n, n + r - 1)
 
-        for i in range(0, r + 1):
-            a = Mul(*[ A.subs(n, n + j) for j in range(0, i) ])
-            b = Mul(*[ B.subs(n, n + j) for j in range(i, r) ])
+        for i in range(r + 1):
+            a = Mul(*[A.subs(n, n + j) for j in range(i)])
+            b = Mul(*[B.subs(n, n + j) for j in range(i, r)])
 
             poly = quo(coeffs[i]*a*b, D, n)
             polys.append(poly.as_poly(n))
@@ -610,7 +610,7 @@ def rsolve_hyper(coeffs, f, n, **hints):
         else:
             return None
 
-        for i in range(0, r + 1):
+        for i in range(r + 1):
             coeff = polys[i].nth(d)
 
             if coeff is not S.Zero:
@@ -620,7 +620,7 @@ def rsolve_hyper(coeffs, f, n, **hints):
             if z.is_zero:
                 continue
 
-            (C, s) = rsolve_poly([ polys[i]*z**i for i in range(r + 1) ], 0, n, symbols=True)
+            (C, s) = rsolve_poly([polys[i]*z**i for i in range(r + 1)], 0, n, symbols=True)
 
             if C is not None and C is not S.Zero:
                 symbols |= set(s)
@@ -676,12 +676,12 @@ def rsolve(f, y, init=None):
 
     Initial conditions can be given as a dictionary in two forms:
 
-        (1) ``{   n_0  : v_0,   n_1  : v_1, ...,   n_m  : v_m }``
-        (2) ``{ y(n_0) : v_0, y(n_1) : v_1, ..., y(n_m) : v_m }``
+        (1) ``{  n_0  : v_0,   n_1  : v_1, ...,   n_m  : v_m}``
+        (2) ``{y(n_0) : v_0, y(n_1) : v_1, ..., y(n_m) : v_m}``
 
     or as a list ``L`` of values:
 
-        ``L = [ v_0, v_1, ..., v_m ]``
+        ``L = [v_0, v_1, ..., v_m]``
 
     where ``L[i] = v_i``, for `i=0, \ldots, m`, maps to `y(n_i)`.
 
@@ -702,7 +702,7 @@ def rsolve(f, y, init=None):
     >>> rsolve(f, y(n))
     2**n*C0 + C1*factorial(n)
 
-    >>> rsolve(f, y(n), { y(0):0, y(1):3 })
+    >>> rsolve(f, y(n), {y(0):0, y(1):3})
     3*2**n - 3*factorial(n)
 
     See Also
@@ -735,7 +735,7 @@ def rsolve(f, y, init=None):
                         kspec = int(result[k])
                     else:
                         raise ValueError(
-                            "'%s(%s+k)' expected, got '%s'" % (y.func, n, h))
+                            "'%s(%s + k)' expected, got '%s'" % (y.func, n, h))
                 else:
                     raise ValueError(
                         "'%s' expected, got '%s'" % (y.func, h.func))
@@ -800,7 +800,7 @@ def rsolve(f, y, init=None):
         init = None
 
     if symbols and init is not None:
-        if type(init) is list:
+        if isinstance(init, list):
             init = {i: init[i] for i in range(len(init))}
 
         equations = []
