@@ -1,8 +1,8 @@
 from sympy.core.assumptions import StdFactKB
-from sympy.core import S, Pow, Symbol
-from sympy.core.expr import AtomicExpr
-from sympy.core.compatibility import range
-from sympy import diff as df, sqrt, ImmutableMatrix as Matrix
+from sympy.core import S, Pow, sympify
+from sympy.core.expr import AtomicExpr, Expr
+from sympy.core.compatibility import range, default_sort_key
+from sympy import sqrt, ImmutableMatrix as Matrix, Add
 from sympy.vector.coordsysrect import CoordSys3D
 from sympy.vector.basisdependent import (BasisDependent, BasisDependentAdd,
                                          BasisDependentMul, BasisDependentZero)
@@ -60,7 +60,7 @@ class Vector(BasisDependent):
         expression).
         If 'other' is a Dyadic, the dot product is returned as a Vector.
         If 'other' is an instance of Del, returns the directional
-        derivate operator as a Python function. If this function is
+        derivative operator as a Python function. If this function is
         applied to a scalar expression, it returns the directional
         derivative of the scalar field wrt this Vector.
 
@@ -91,7 +91,6 @@ class Vector(BasisDependent):
 
         """
 
-        from sympy.vector.functions import express
         # Check special cases
         if isinstance(other, Dyadic):
             if isinstance(self, VectorZero):
@@ -109,33 +108,11 @@ class Vector(BasisDependent):
         # Check if the other is a del operator
         if isinstance(other, Del):
             def directional_derivative(field):
-                from sympy.vector.operators import _get_coord_sys_from_expr
-                coord_sys = _get_coord_sys_from_expr(field)
-                if coord_sys is not None:
-                    field = express(field, coord_sys, variables=True)
-                    out = self.dot(coord_sys._i) * df(field, coord_sys._x)
-                    out += self.dot(coord_sys._j) * df(field, coord_sys._y)
-                    out += self.dot(coord_sys._k) * df(field, coord_sys._z)
-                    if out == 0 and isinstance(field, Vector):
-                        out = Vector.zero
-                    return out
-                elif isinstance(field, Vector) :
-                    return Vector.zero
-                else:
-                    return S(0)
+                from sympy.vector.functions import directional_derivative
+                return directional_derivative(field, self)
             return directional_derivative
 
-        if isinstance(self, VectorZero) or isinstance(other, VectorZero):
-            return S(0)
-
-        v1 = express(self, other._sys)
-        v2 = express(other, other._sys)
-        dotproduct = S(0)
-        for x in other._sys.base_vectors():
-            dotproduct += (v1.components.get(x, 0) *
-                           v2.components.get(x, 0))
-
-        return dotproduct
+        return dot(self, other)
 
     def __and__(self, other):
         return self.dot(other)
@@ -183,39 +160,8 @@ class Vector(BasisDependent):
                 outer = cross_product.outer(k.args[1])
                 outdyad += v * outer
             return outdyad
-        elif not isinstance(other, Vector):
-            raise TypeError(str(other) + " is not a vector")
-        elif (isinstance(self, VectorZero) or
-                isinstance(other, VectorZero)):
-            return Vector.zero
 
-        # Compute cross product
-        def _det(mat):
-            """This is needed as a little method for to find the determinant
-            of a list in python.
-            SymPy's Matrix won't take in Vector, so need a custom function.
-            The user shouldn't be calling this.
-
-            """
-
-            return (mat[0][0] * (mat[1][1] * mat[2][2] - mat[1][2] *
-                                 mat[2][1]) +
-                    mat[0][1] * (mat[1][2] * mat[2][0] - mat[1][0] *
-                                 mat[2][2]) +
-                    mat[0][2] * (mat[1][0] * mat[2][1] - mat[1][1] *
-                                 mat[2][0]))
-
-        outvec = Vector.zero
-        for system, vect in other.separate().items():
-            tempi = system.i
-            tempj = system.j
-            tempk = system.k
-            tempm = [[tempi, tempj, tempk],
-                     [self & tempi, self & tempj, self & tempk],
-                     [vect & tempi, vect & tempj, vect & tempk]]
-            outvec += _det(tempm)
-
-        return outvec
+        return cross(self, other)
 
     def __xor__(self, other):
         return self.cross(other)
@@ -287,6 +233,34 @@ class Vector(BasisDependent):
         else:
             return self.dot(other) / self.dot(self) * self
 
+    @property
+    def _projections(self):
+        """
+        Returns the components of this vector but the output includes
+        also zero values components.
+
+        Examples
+        ========
+
+        >>> from sympy.vector import CoordSys3D, Vector
+        >>> C = CoordSys3D('C')
+        >>> v1 = 3*C.i + 4*C.j + 5*C.k
+        >>> v1._projections
+        (3, 4, 5)
+        >>> v2 = C.x*C.y*C.z*C.i
+        >>> v2._projections
+        (C.x*C.y*C.z, 0, 0)
+        >>> v3 = Vector.zero
+        >>> v3._projections
+        (0, 0, 0)
+        """
+
+        from sympy.vector.operators import _get_coord_sys_from_expr
+        if isinstance(self, VectorZero):
+            return (S(0), S(0), S(0))
+        base_vec = next(iter(_get_coord_sys_from_expr(self))).base_vectors()
+        return tuple([self.dot(i) for i in base_vec])
+
     def __or__(self, other):
         return self.outer(other)
 
@@ -356,8 +330,11 @@ class BaseVector(Vector, AtomicExpr):
 
     """
 
-    def __new__(cls, name, index, system, pretty_str, latex_str):
-        name = str(name)
+    def __new__(cls, index, system, pretty_str=None, latex_str=None):
+        if pretty_str is None:
+            pretty_str = "x{0}".format(index)
+        if latex_str is None:
+            latex_str = "x_{0}".format(index)
         pretty_str = str(pretty_str)
         latex_str = str(latex_str)
         # Verify arguments
@@ -365,15 +342,14 @@ class BaseVector(Vector, AtomicExpr):
             raise ValueError("index must be 0, 1 or 2")
         if not isinstance(system, CoordSys3D):
             raise TypeError("system should be a CoordSys3D")
+        name = system._vector_names[index]
         # Initialize an object
-        obj = super(BaseVector, cls).__new__(cls, Symbol(name), S(index),
-                                             system, Symbol(pretty_str),
-                                             Symbol(latex_str))
+        obj = super(BaseVector, cls).__new__(cls, S(index), system)
         # Assign important attributes
         obj._base_instance = obj
         obj._components = {obj: S(1)}
         obj._measure_number = S(1)
-        obj._name = name
+        obj._name = system._name + '.' + name
         obj._pretty_form = u'' + pretty_str
         obj._latex_form = latex_str
         obj._system = system
@@ -444,7 +420,7 @@ class VectorMul(BasisDependentMul, Vector):
 
     @property
     def measure_number(self):
-        """ The scalar expression involved in the defition of
+        """ The scalar expression involved in the definition of
         this VectorMul.
         """
         return self._measure_number
@@ -457,13 +433,164 @@ class VectorZero(BasisDependentZero, Vector):
 
     _op_priority = 12.1
     _pretty_form = u'0'
-    _latex_form = '\mathbf{\hat{0}}'
+    _latex_form = r'\mathbf{\hat{0}}'
 
     def __new__(cls):
         obj = BasisDependentZero.__new__(cls)
         return obj
 
 
+class Cross(Vector):
+    """
+    Represents unevaluated Cross product.
+
+    Examples
+    ========
+
+    >>> from sympy.vector import CoordSys3D, Cross
+    >>> R = CoordSys3D('R')
+    >>> v1 = R.i + R.j + R.k
+    >>> v2 = R.x * R.i + R.y * R.j + R.z * R.k
+    >>> Cross(v1, v2)
+    Cross(R.i + R.j + R.k, R.x*R.i + R.y*R.j + R.z*R.k)
+    >>> Cross(v1, v2).doit()
+    (-R.y + R.z)*R.i + (R.x - R.z)*R.j + (-R.x + R.y)*R.k
+
+    """
+
+    def __new__(cls, expr1, expr2):
+        expr1 = sympify(expr1)
+        expr2 = sympify(expr2)
+        if default_sort_key(expr1) > default_sort_key(expr2):
+            return -Cross(expr2, expr1)
+        obj = Expr.__new__(cls, expr1, expr2)
+        obj._expr1 = expr1
+        obj._expr2 = expr2
+        return obj
+
+    def doit(self, **kwargs):
+        return cross(self._expr1, self._expr2)
+
+
+class Dot(Expr):
+    """
+    Represents unevaluated Dot product.
+
+    Examples
+    ========
+
+    >>> from sympy.vector import CoordSys3D, Dot
+    >>> from sympy import symbols
+    >>> R = CoordSys3D('R')
+    >>> a, b, c = symbols('a b c')
+    >>> v1 = R.i + R.j + R.k
+    >>> v2 = a * R.i + b * R.j + c * R.k
+    >>> Dot(v1, v2)
+    Dot(R.i + R.j + R.k, a*R.i + b*R.j + c*R.k)
+    >>> Dot(v1, v2).doit()
+    a + b + c
+
+    """
+
+    def __new__(cls, expr1, expr2):
+        expr1 = sympify(expr1)
+        expr2 = sympify(expr2)
+        expr1, expr2 = sorted([expr1, expr2], key=default_sort_key)
+        obj = Expr.__new__(cls, expr1, expr2)
+        obj._expr1 = expr1
+        obj._expr2 = expr2
+        return obj
+
+    def doit(self, **kwargs):
+        return dot(self._expr1, self._expr2)
+
+
+def cross(vect1, vect2):
+    """
+    Returns cross product of two vectors.
+
+    Examples
+    ========
+
+    >>> from sympy.vector import CoordSys3D
+    >>> from sympy.vector.vector import cross
+    >>> R = CoordSys3D('R')
+    >>> v1 = R.i + R.j + R.k
+    >>> v2 = R.x * R.i + R.y * R.j + R.z * R.k
+    >>> cross(v1, v2)
+    (-R.y + R.z)*R.i + (R.x - R.z)*R.j + (-R.x + R.y)*R.k
+
+    """
+    if isinstance(vect1, Add):
+        return VectorAdd.fromiter(cross(i, vect2) for i in vect1.args)
+    if isinstance(vect2, Add):
+        return VectorAdd.fromiter(cross(vect1, i) for i in vect2.args)
+    if isinstance(vect1, BaseVector) and isinstance(vect2, BaseVector):
+        if vect1._sys == vect2._sys:
+            n1 = vect1.args[0]
+            n2 = vect2.args[0]
+            if n1 == n2:
+                return Vector.zero
+            n3 = ({0,1,2}.difference({n1, n2})).pop()
+            sign = 1 if ((n1 + 1) % 3 == n2) else -1
+            return sign*vect1._sys.base_vectors()[n3]
+        try:
+            from .functions import express
+            return cross(express(vect1, vect2._sys), vect2)
+        except:
+            return Cross(vect1, vect2)
+    if isinstance(vect1, VectorZero) or isinstance(vect2, VectorZero):
+        return Vector.zero
+    if isinstance(vect1, VectorMul):
+        v1, m1 = next(iter(vect1.components.items()))
+        return m1*cross(v1, vect2)
+    if isinstance(vect2, VectorMul):
+        v2, m2 = next(iter(vect2.components.items()))
+        return m2*cross(vect1, v2)
+
+    return Cross(vect1, vect2)
+
+
+def dot(vect1, vect2):
+    """
+    Returns dot product of two vectors.
+
+    Examples
+    ========
+
+    >>> from sympy.vector import CoordSys3D
+    >>> from sympy.vector.vector import dot
+    >>> R = CoordSys3D('R')
+    >>> v1 = R.i + R.j + R.k
+    >>> v2 = R.x * R.i + R.y * R.j + R.z * R.k
+    >>> dot(v1, v2)
+    R.x + R.y + R.z
+
+    """
+    if isinstance(vect1, Add):
+        return Add.fromiter(dot(i, vect2) for i in vect1.args)
+    if isinstance(vect2, Add):
+        return Add.fromiter(dot(vect1, i) for i in vect2.args)
+    if isinstance(vect1, BaseVector) and isinstance(vect2, BaseVector):
+        if vect1._sys == vect2._sys:
+            return S.One if vect1 == vect2 else S.Zero
+        try:
+            from .functions import express
+            return dot(vect1, express(vect2, vect1._sys))
+        except:
+            return Dot(vect1, vect2)
+    if isinstance(vect1, VectorZero) or isinstance(vect2, VectorZero):
+        return S.Zero
+    if isinstance(vect1, VectorMul):
+        v1, m1 = next(iter(vect1.components.items()))
+        return m1*dot(v1, vect2)
+    if isinstance(vect2, VectorMul):
+        v2, m2 = next(iter(vect2.components.items()))
+        return m2*dot(vect1, v2)
+
+    return Dot(vect1, vect2)
+
+
 def _vect_div(one, other):
     """ Helper for division involving vectors. """
     if isinstance(one, Vector) and isinstance(other, Vector):
